/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
Triangle Solver

Author: Ryan Cumings <rcumings@clckwcher.ath.cx>
Created: 2004-05-23
Version: 1.00
Change Log:
	2004-05-23
	- Initial release after working on it for the weekend
	  should be buggless
**/

#define USE_TI89              // Compile for TI-89

//#define USE_KERNEL            // Make Kernel Program

#define RETURN_VALUE          // Return Pushed Expression

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define MIN_AMS 200           // Compile for AMS 2.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#define VALUE_LEN 30

#define ANGLE_X 0x01
#define ANGLE_Y 0x02
#define ANGLE_Z 0x04
#define SIDE_X 0x10
#define SIDE_Y 0x20
#define SIDE_Z 0x40

// Handle Declarations
HANDLE menu_handle;
HANDLE exec_handle;
short key_result;
void *kbq;
const char *err_string;

LCD_BUFFER UI;

char tri_name[4];
int state;
int prep_state;
char value[VALUE_LEN];
char enteredTypes;
char currentType;
double angles[6];
double lengths[6];
double areas[2];

BOOL radians;

void setup_program(void);
void get_tri_name(char t);
void check_tri_name(char t);
void select_info(char t);
void enter_info(char t);
void compute_info(char t);
void angle_type(char t);
void display_info(char t);
void display_am_info(char t);
void draw_triangle(double * pLengths, double * pAngles, char * pName);

// Main Function
void _main(void)
{	
	// Initialize Program
	setup_program();
	
	while(TRUE) {
		// If queue was empty, set the incoming key to 0
		if(OSdequeue (&key_result, kbq)) {
			key_result = 0;
		}
		
		// Is this state prep'ed?
		if(prep_state != state) {
			switch(state) {
				case 0:
				case 1:
				case 2: {
					get_tri_name(1);
					break;
				}
				case 3: {
					check_tri_name(1);
					break;
				}
				case 4: {
					angle_type(1);
					break;
				}
				// Why bother making a bunch of code for this?
				case 5: {
					int i;
					for(i = 0; i < 6; i ++) {
						angles[i] = 0;
						lengths[i] = 0;
					}
					state ++;
					// Don't break cause we can prep for next state
				}
				// At state 6 we clear the entered types
				case 6: {
					currentType = 0;
					enteredTypes = 0;
				}
				case 8:
				case 10: {
					select_info(1);
					break;
				}
				case 7:
				case 9:
				case 11: {
					enter_info(1);
					break;
				}
				// State 12 doesn't need any prep work
//				case 12: {
//					compute_info(1);
//					break;
//				}
				case 13: {
					display_info(1);
					break;
				}
				case 14: {
					display_am_info(1);
					break;
				}
				default: {
					break;
				}
			}
			prep_state = state;
		}
		
		// All ways to quit the program
		if(key_result == KEY_F1 || key_result == KEY_QUIT || key_result == KEY_OFF || key_result == KEY_ON ) {
			goto DONE;
		}
		// Restart the program
		else if(key_result == KEY_F2) {
			prep_state = -1;
			state = 0;
		}
		// Just go back to entering data
		// Make sure that we are above state 6 before we do this
		else if((key_result == KEY_F3) && (state > 5)) {
			prep_state = -1;
			state = 6;
		}
		// State machine
		switch(state) {
			case 0:
			case 1:
			case 2: {
				get_tri_name(0);
				break;
			}
			case 3: {
				check_tri_name(0);
				break;
			}
			case 4: {
				angle_type(0);
				break;
			}
			case 6:
			case 8:
			case 10: {
				select_info(0);
				break;
			}
			case 7:
			case 9:
			case 11: {
				enter_info(0);
				break;
			}
			case 12: {
				compute_info(0);
				break;
			}
			// We don't need state 13 because display_info doesn't take user input
//			case 13: {
//				display_info(0);
//				break;
//			}
			case 14: {
				display_am_info(0);
				break;
			}
			default: {
				break;
			}
		}
	}
	
DONE:
	
	push_quantum(0xE9);
}

/****************************
* Get Triangle Name
* 
* Let the user enter their own letters
****************************/
void get_tri_name(char t) {
	if(t == 1) {
		// Prep this so we erase the current point
		tri_name[state] = 0;
		// Make sure we have a zero terminate string
		tri_name[state + 1] = 0;
		// Obviously we only allow letters, so why not just set this
		SetAlphaStatus(TRUE);
		goto UPDATE_GET_TRI_NAME;
	}
	switch(key_result) {
		case 0: {
			return;
		}
		// We can return for these because the key is obviously not
		// A letter
		case KEY_BACKSPACE: {
			state --;
			return;
		}
		case KEY_CLEAR: {
			state = 0;
			return;
		}
	}
	// Is that user being tricky?
	if(!isalpha(key_result)) {
		return;
	}
	
	// Make sure this letter isn't already taken
	if((tri_name[0] == key_result) && (state > 0)) {
		return;
	}
	else if((tri_name[1] == key_result) && (state > 1)) {
		return;
	}
	// We cleared the letter so it can be used
	else {
		tri_name[state] = key_result;
		state ++;
	}
UPDATE_GET_TRI_NAME:	
	LCD_restore(&UI);
	printf_xy(0,20,"Enter triangle name: %s", tri_name);
	DrawChar(strlen(tri_name) * 6 + 128, 20, ' ',A_REVERSE);
}

/****************************
* Check Triangle Name
* 
* Make sure they didn't enter the wrong name
****************************/
void check_tri_name(char t) {
	// Save the cursor state
	static BOOL yes;
	if(t == 1) {
		yes = TRUE;
		goto UPDATE_CHECK_TRI_NAME;
	}
	switch(key_result) {
		// Any of these keys will change selection
		case KEY_UP:
		case KEY_DOWN:
		case KEY_LEFT:
		case KEY_RIGHT: {
			yes = !yes;
			break;
		}
		case KEY_ENTER: {
			if(yes) {
				state ++;
			}
			else {
				state = 0;
			}
			break;
		}
		default: {
			return;
		}
	}
UPDATE_CHECK_TRI_NAME:
	LCD_restore(&UI);
	printf_xy(0,20,"Is this correct? %s",tri_name);
	DrawStr(20,40,"YES",( yes ? A_REVERSE : A_NORMAL ));
	DrawStr(46,40,"NO",( yes ? A_NORMAL : A_REVERSE ));
}

/****************************
* Angle Type
* 
* User selects which angle units to use
* Angles are stored internally as radians regardless
* of selection here
****************************/
void angle_type(char t) {
	if(t == 1) {
		// Don't initialize global variable radians because we might
		// want to use the same type again
		goto UPDATE_ANGLE_TYPE;
	}
	switch(key_result) {
		// Any of these keys will change selection
		case KEY_UP:
		case KEY_DOWN:
		case KEY_LEFT:
		case KEY_RIGHT: {
			radians = !radians;
			break;
		}
		case KEY_ENTER: {
			state ++;
			break;
		}
		default: {
			return;
		}
	}
	
UPDATE_ANGLE_TYPE:
	LCD_restore(&UI);
	printf_xy(0,20,"Choose Angle Type");
	DrawStr(0,40,"DEGREES",( radians ? A_NORMAL : A_REVERSE ));
	DrawStr(48,40,"RADIANS",( radians ? A_REVERSE : A_NORMAL ));
}

/****************************
* Select Info
* 
* User selects the piece to enter
****************************/
void select_info(char t) {
	if(t == 1) {
		goto UPDATE_SELECT_INFO;
	}
	
	switch(key_result) {
		case KEY_ENTER: {
			state ++;
			break;
		}
		case KEY_DOWN:
		case KEY_UP: {
			currentType = (currentType + 0x10) & 0x13;
			break;
		}
		case KEY_LEFT: {
			if((currentType & 0x03) == 0) {
				currentType = (currentType & 0x10) + 0x02;
			}
			else {
				currentType --;
			}
			break;
		}
		case KEY_RIGHT: {
			if((currentType & 0x03) < 2) {
				currentType ++;
			}
			else {
				currentType = (currentType & 0x10);
			}
			break;
		}
		case KEY_BACKSPACE:
		case KEY_CLEAR: {
			if((currentType & 0x10) != 0) {
				if((enteredTypes & (ANGLE_X << (currentType & 0x03))) != 0) {
					// It's ok to do this because we need to be at least enough states to get here
					state -= 2;
					enteredTypes &= 0xFF ^ (ANGLE_X << (currentType & 0x03));
				}
			}
			else {
				if((enteredTypes & (SIDE_X << (currentType & 0x03))) != 0) {
					// It's ok to do this because we need to be at least enough states to get here
					state -= 2;
					enteredTypes &= 0xFF ^ (SIDE_X << (currentType & 0x03));
				}
			}
			break;
		}
		default: {
			return;
		}
	}
	
UPDATE_SELECT_INFO:
	LCD_restore(&UI);
	printf_xy(42,20,"%c %c %c", tri_name[0], tri_name[1], tri_name[2]);
	printf_xy(0,28,"SIDE:");
	printf_xy(0,36,"ANGLE:");
	DrawChar(42,28,((enteredTypes & SIDE_X) != 0 ? 'X' : ' '), ( (currentType == 0x00) != 0 ? A_REVERSE : A_NORMAL ) );
	DrawChar(54,28,((enteredTypes & SIDE_Y) != 0 ? 'X' : ' '), ( (currentType == 0x01) != 0 ? A_REVERSE : A_NORMAL ) );
	DrawChar(66,28,((enteredTypes & SIDE_Z) != 0 ? 'X' : ' '), ( (currentType == 0x02) != 0 ? A_REVERSE : A_NORMAL ) );
	DrawChar(42,36,((enteredTypes & ANGLE_X) != 0 ? 'X' : ' '), ( (currentType == 0x10) != 0 ? A_REVERSE : A_NORMAL ) );
	DrawChar(54,36,((enteredTypes & ANGLE_Y) != 0 ? 'X' : ' '), ( (currentType == 0x11) != 0 ? A_REVERSE : A_NORMAL ) );
	DrawChar(66,36,((enteredTypes & ANGLE_Z) != 0 ? 'X' : ' '), ( (currentType == 0x12) != 0 ? A_REVERSE : A_NORMAL ) );
	
	// Display currently selected piece
	if((currentType & 0x10) != 0) {
		// Make sure we display in selected angle type
		printf_xy(0,48,"%f",angles[currentType & 0x3] * ( radians ? 1 : 180 / PI ) );
	}
	else {
		printf_xy(0,48,"%f",lengths[currentType & 0x3]);
	}
}

/****************************
* Enter Info
* 
* User enters the value or expression for the current piece
****************************/
void enter_info(char t) {
	double nValue;
	static int curr_len;
	
	if(t == 1) {
		SetAlphaStatus(FALSE);
		int i;
		// Reset our parse string
		for(i = 0; i < VALUE_LEN; i ++) {
			value[i] = 0;
		}
		
		// Read current value from memory, if we decide to change it
		if((currentType & 0x10) != 0) {
			curr_len = sprintf(value,"%f",angles[currentType & 0x3] * ( radians ? 1 : 180 / PI ) );
		}
		else {
			curr_len = sprintf(value,"%f",lengths[currentType & 0x3]);
		}
		goto UPDATE_ENTER_INFO;
	}
	
	err_string = "Generic";
	
	switch(key_result) {
		case KEY_ENTER: {
	  TRY
			err_string = "Can not be parsed";
			push_parse_text(value);
			if(! can_be_approxed (top_estack, FALSE) ) {
		    err_string = "Can not be approxed";
	  	  goto ERR_PAUSE;
			}
			err_string = "Can not be simp'ed";
			push_internal_simplify(top_estack);
			err_string = "Approximation"; 
			NG_approxESI (top_estack);
			err_string = "Expr conversion";
			nValue = estack_number_to_Float (top_estack);
	  ONERR
  		goto ERR_PAUSE;
	  ENDTRY
			if((currentType & 0x10) != 0) {
				// Update value
				// Store the angle in radians for speed
				if(radians) {
					angles[currentType & 0x3] = nValue;
				}
				else {
					angles[currentType & 0x3] = nValue * PI / 180;
				}
				// If we already have this entered, don't count as another piece
				if((enteredTypes & (ANGLE_X << (currentType & 0x3))) != 0) {
					state --;
					break;
				}
				// Count this as a piece
				enteredTypes |= (ANGLE_X << (currentType & 0x3));
			}
			else {
				// Update value
				lengths[currentType & 0x3] = nValue;
				// If we already have this entered, don't count as another piece
				if((enteredTypes & (SIDE_X << (currentType & 0x3))) != 0) {
					state --;
					break;
				}
				// Count this as a piece
				enteredTypes |= (SIDE_X << (currentType & 0x3));
			}
			state ++;
			break;
		}
		// Square Root Symbol
		case 0x102A: {
			if(curr_len < VALUE_LEN - 2) {
				value[curr_len ++] = 0xA8;
				value[curr_len ++] = '(';
			}
			break;
		}
		// Integrate Symbol
		case 0x1037: {
			if(curr_len < VALUE_LEN - 2) {
				value[curr_len ++] = 0xBD;
				value[curr_len ++] = '(';
			}
			break;
		}
		// Various functions accessable from keyboard
		case 0x1058: {
			if(curr_len < VALUE_LEN - 3) {
				value[curr_len ++] = 'l';
				value[curr_len ++] = 'n';
				value[curr_len ++] = '(';
			}
			break;
		}
		case 0x4058: {
			if(curr_len < VALUE_LEN - 3) {
				value[curr_len ++] = 'e';
				value[curr_len ++] = '^';
				value[curr_len ++] = '(';
			}
			break;
		}
		case 0x1059: {
			if(curr_len < VALUE_LEN - 4) {
				value[curr_len ++] = 's';
				value[curr_len ++] = 'i';
				value[curr_len ++] = 'n';
				value[curr_len ++] = '(';
			}
			break;
		}
		case 0x4059: {
			if(curr_len < VALUE_LEN - 5) {
				value[curr_len ++] = 's';
				value[curr_len ++] = 'i';
				value[curr_len ++] = 'n';
				value[curr_len ++] = 0xB4;
				value[curr_len ++] = '(';
			}
			break;
		}
		case 0x105A: {
			if(curr_len < VALUE_LEN - 4) {
				value[curr_len ++] = 'c';
				value[curr_len ++] = 'o';
				value[curr_len ++] = 's';
				value[curr_len ++] = '(';
			}
			break;
		}
		case 0x405A: {
			if(curr_len < VALUE_LEN - 5) {
				value[curr_len ++] = 'c';
				value[curr_len ++] = 'o';
				value[curr_len ++] = 's';
				value[curr_len ++] = 0xB4;
				value[curr_len ++] = '(';
			}
		}
		case 0x1054: {
			if(curr_len < VALUE_LEN - 4) {
				value[curr_len ++] = 't';
				value[curr_len ++] = 'a';
				value[curr_len ++] = 'n';
				value[curr_len ++] = '(';
			}
			break;
		}
		case 0x4054: {
			if(curr_len < VALUE_LEN - 5) {
				value[curr_len ++] = 't';
				value[curr_len ++] = 'a';
				value[curr_len ++] = 'n';
				value[curr_len ++] = 0xB4;
				value[curr_len ++] = '(';
			}
			break;
		}
		case KEY_BACKSPACE: {
			// Don't backspace unless we actually can
			if(curr_len >= 1) {
				value[-- curr_len] = 0;
			}
			break;
		}
		case KEY_CLEAR: {
			int i;
			// Reset our parse string
			for(i = 0; i < VALUE_LEN; i ++) {
				value[i] = 0;
			}
			curr_len = 0;
			break;
		}
		case 0: {
			return;
		}
		// This key hasn't been intercepted so it probably is good
		default: {
			// Don't overflow our buffer
			if(curr_len + 1 < VALUE_LEN) {
				value[curr_len ++] = key_result;
			}
			break;
		}
	}
	
UPDATE_ENTER_INFO:
	LCD_restore(&UI);
	printf_xy(0,20,"%s",value);
	DrawChar(curr_len * 6 + 2, 20, ' ',A_REVERSE);
	
	return;
	
ERR_PAUSE:
  printf_xy (0, 78, "Error: %s", err_string);
	push_END_TAG ();
	cmd_pause (top_estack);
}

/****************************
* Compute Info
* 
* User enters the value or expression for the current piece
****************************/
void compute_info(char t) {
	unsigned char nAngles;
	double temp;
	
	// Count our angles
	nAngles = 0;
	if((enteredTypes & ANGLE_X) != 0) {
		nAngles ++;
	}
	if((enteredTypes & ANGLE_Y) != 0) {
		nAngles ++;
	}
	if((enteredTypes & ANGLE_Z) != 0) {
		nAngles ++;
	}
	switch(nAngles) {
		// Case is SSS
		case 0: {
			// Make sure no length is 0
			if(lengths[0] == 0 || lengths[1] == 0 || lengths[2] == 0) {
				err_string = "A length is 0";
				goto ERR_PAUSE;
			}
			// Compute the angles
			int i;
			for(i = 0; i < 3; i++) {
				temp = (lengths[(2 + i) % 3] * lengths[(2 + i) % 3] +  lengths[(1 + i) % 3] * lengths[(1 + i) % 3] - lengths[i] * lengths[i]) / ( 2 * lengths[(2 + i) % 3] * lengths[(1 + i) % 3] );
				// We can't take acos of a number out of the range of cos
				if(temp > 1 || temp < -1) {
					err_string = "Not a triangle";
					goto ERR_PAUSE;
				}
				angles[i] = acos(temp);
			}
			// Just use any sides because all are given
			areas[0] = lengths[0] * lengths[1] * sin(2) / 2;
			break;
		}
		// Case is SAS | SSA
		case 1: {
			int i,j,k,l,m;
			// Find the missing side
			for(i = 0; i < 3; i++) {
				if((enteredTypes & (SIDE_X << i)) == 0) {
					// Find the given angle
					for(j = 0; j < 3; j++) {
						if((enteredTypes & (ANGLE_X << j)) != 0) {
							// These are the two sides we know
							l = (i + 1) % 3;
							m = (i + 2) % 3;
							// We have SAS, no ambiguous case
							if(i == j) {
SAS:
								lengths[i] = sqrt(lengths[l] * lengths[l] + lengths[m] * lengths[m] - 2 * lengths[l] * lengths[m] * cos(angles[i]));
								// We already have SAS so just plug in to compute area
								areas[0] = lengths[l] * lengths[m] * sin(i) / 2;
								
								// We can only get here if we went by the label
								// Besides why bother doing all the angles again
								if(i != j) {
									break;
								}
								// Find Unknown Angles
								temp = lengths[m] * sin(angles[i]) / lengths[i];
								// Make sure we have a triangle
								if(temp > 1) {
									err_string = "Not a triangle";
									goto ERR_PAUSE;
								}
								angles[m] = asin(temp);
								temp = lengths[l] * sin(angles[i]) / lengths[i];
								// Make sure we have a triangle
								if(temp > 1) {
									err_string = "Not a triangle";
									goto ERR_PAUSE;
								}
								angles[l] = asin(temp);
								break;
							}
							// The k angle is not given, but the j angle is. so use the j law of sines to solve for k
							else if(j == l) {
								k = m;
							}
							else {
								k = l;
							}
							temp = lengths[k] * sin(angles[j]) / lengths[j];
							// Make sure we have a triangle
							if(temp > 1) {
								err_string = "Not a triangle";
								goto ERR_PAUSE;
							}
							angles[k] = asin(temp);
							// Largest Side must be opposite largest angle
							if(((angles[k] > angles[j]) && (lengths[k] < lengths[j])) || ((angles[k] < angles[j]) && (lengths[k] > lengths[j])) ) {
								err_string = "Not a triangle";
								goto ERR_PAUSE;
							}
							// Check if we have an acute triangle
							if(angles[k] + angles[j] >= PI) {
								// Do we at least have an obtuse triangle?
								if(angles[j] >= angles[k]) {
									err_string = "Not a triangle";
									goto ERR_PAUSE;
								}
								// Make the obtuse triangle the normal case and jump to SAS
								// Optimization
								angles[i] = angles[k] - angles[j];
								angles[k] = PI - angles[k];
								goto SAS;
							}
							// Check if there is ambiguous case
							angles[i] = PI - angles[k] - angles[j];
							if(angles[j] >= angles[k]) {
								goto SAS;
							}
							// Setup for ambigous case
							angles[j + 3] = angles[j];
							lengths[l + 3] = lengths[l];
							lengths[m + 3] = lengths[m];
							// Solve for ambiguous case
							// Optimization
							angles[i + 3] = angles[k] - angles[j + 3];
							angles[k + 3] = PI - angles[k];
							lengths[i + 3] = sqrt(lengths[l + 3] * lengths[l + 3] + lengths[m + 3] * lengths[m + 3] - 2 * lengths[l + 3] * lengths[m + 3] * cos(angles[i + 3]));
							// We already have SAS so just plug in to compute area
							areas[1] = lengths[l + 3] * lengths[m + 3] * sin(angles[i + 3]) / 2;
							// We increment here so we can use the ambiguous display
							state ++;
							goto SAS;
						}
					}
					break;
				}
			}
			
			break;
		}
		// Case is AAS | ASA
		// Just solve for third angle and treat as ASA
		case 2: {
			int i;
			// Find the missing angle
			for(i = 0; i < 3; i++) {
				if((enteredTypes & (ANGLE_X << i)) == 0) {
					angles[i] = PI - angles[(1 + i) % 3] - angles[(2 + i) % 3];
					if(angles[i] <= 0) {
						err_string = "Not a triangle";
						goto ERR_PAUSE;
					}
					break;
				}
			}
			
			// Find the given side
			for(i = 0; i < 3; i++) {
				if((enteredTypes & (SIDE_X << i)) != 0) {
					lengths[(i + 1) % 3] = (lengths[i] / sin(angles[i])) * sin(angles[(i + 1) % 3]);
					lengths[(i + 2) % 3] = (lengths[i] / sin(angles[i])) * sin(angles[(i + 2) % 3]);
					break;
				}
			}
			
			// Use the given side to compute area
			areas[0] = lengths[i] * lengths[(i + 1) % 3] * sin((i + 2) % 3) / 2;
			break;
		}
		// Case is AAA
		// Best we can do is just compute ratios
		case 3: {
			temp = (angles[0] + angles[1] + angles[2]);
			// Make sure the user actually gave us a triangle
			// Range tolerance because we may have not a perfect conversion
			if(temp < (PI - 0.0001) || temp > (PI + 0.0001) ) {
				err_string = "Not a triangle";
				goto ERR_PAUSE;
			}
			
			// Find the smallest angle, therefore the shortest side
			int i;
			i = ( angles[1] < angles[0] ? 1 : 0 );
			i = ( angles[2] < angles[i] ? 2 : i );
			
			// Shortest side will be the base for the ratio
			lengths[i] = 1;
			
			// Solve like ASA
			lengths[(i + 1) % 3] = (lengths[i] / sin(angles[i])) * sin(angles[(i + 1) % 3]);
			lengths[(i + 2) % 3] = (lengths[i] / sin(angles[i])) * sin(angles[(i + 2) % 3]);
			
			// Use the smallest side for area because we know its 1
			areas[0] = lengths[(i + 1) % 3] * sin((i + 2) % 3) / 2;
			break;
		}
		// Ouch, we shouldn't be here
		default: {
			err_string = "I'm Deaf and Dumb";
			goto ERR_PAUSE;		
		}
	}
	
	state ++;
	
	return;
	
ERR_PAUSE:
  printf_xy (0, 78, "Error: %s", err_string);
	push_END_TAG ();
	cmd_pause (top_estack);
	// Something we terribly wrong, so let the user re-enter the data
	state = 6;
}

void display_info(char t) {
	
	LCD_restore(&UI);
	printf_xy(12,20,"Side      Angle");
	int i;
	for(i = 0; i < 3; i++) {
		printf_xy(0,28 + i * 8,"%c %-9.3g %-9.9g", tri_name[i], lengths[i], angles[i] * ( radians ? 1 : 180 / PI ) );
	}
	printf_xy(0,52,"Area: %g", areas[0]);
	
	draw_triangle(&lengths[0], &angles[0], tri_name);
}

void display_am_info(char t) {
	static BOOL first_tri;
	if(t == 1) {
		first_tri = TRUE;
		goto UPDATE_DISPLAY_AM_INFO;
	}
	switch(key_result) {
		// All these keys toggle ambiguous case
		case KEY_UP:
		case KEY_DOWN:
		case KEY_LEFT:
		case KEY_RIGHT:
		case KEY_ENTER: {
			first_tri = !first_tri;
			break;
		}
		default: {
			return;
		}
	}
	
UPDATE_DISPLAY_AM_INFO:
	LCD_restore(&UI);
	printf_xy(102,4,( first_tri ? "1st Case" : "2nd Case" ) );
	if(first_tri) {
		printf_xy(12,20,"Side      Angle");
		int i;
		for(i = 0; i < 3; i++) {
			printf_xy(0,28 + i * 8,"%c %-9.3g %-9.9g", tri_name[i], lengths[i], angles[i] * ( radians ? 1 : 180 / PI ) );
		}
		printf_xy(0,52,"Area: %g", areas[0]);
		draw_triangle(&lengths[0], &angles[0], tri_name);
	}
	else {
		printf_xy(12,20,"Side      Angle");
		int i;
		for(i = 0; i < 3; i++) {
			printf_xy(0,28 + i * 8,"%c %-9.3g %-9.9g", tri_name[i], lengths[i + 3], angles[i + 3] * ( radians ? 1 : 180 / PI ) );
		}
		printf_xy(0,52,"Area: %g", areas[1]);
		draw_triangle(&lengths[3], &angles[3], tri_name);
	}
}

void draw_triangle(double * pLengths, double * pAngles, char * pName) {
	int i,j,k;
	
	// Find the smallest side
	i = ( pAngles[1] < pAngles[0] ? 1 : 0 );
	i = ( pAngles[2] < pAngles[i] ? 2 : i );
	
	if(pAngles[(i + 1) % 3] > pAngles[(i + 2) % 3]) {
		j = (i + 1) % 3;
		k = (i + 2) % 3;
	}
	else {
		k = (i + 1) % 3;
		j = (i + 2) % 3;
	}
	
	SCR_RECT area;
	area.xy.x0 = 5;
	area.xy.y0 = 68;
	area.xy.x1 = 155;
	area.xy.y1 = 92;
	
	double minX,maxX,maxY, ratio;
	
	minX = cos(pAngles[i]) * pLengths[j];
	if(minX > 0) {
		if(minX < pLengths[k]) {
			maxX = pLengths[k];
		}
		else {
			maxX = minX;
		}
		minX = 0;
	}
	else {
		maxX = pLengths[k];
	}
	maxY = sin(pAngles[i]) * pLengths[j];
	
	ratio = maxY / (maxX - minX);
	if(ratio > (double) (24) / (double) (150)) {
		ratio = 24 / maxY;
		if(minX < 0) {
			printf_xy( (int) (-minX * ratio) + 5, 92, "%c", pName[j]);
			printf_xy( 5, 60, "%c", pName[k]);
			printf_xy( (int) ((-minX + pLengths[k]) * ratio ) + 5, 92, "%c", pName[i]);
			FillTriangle (5, 68,
										(int) (-minX * ratio) + 5, 92,
										(int) ((-minX + pLengths[k]) * ratio ) + 5, 92, &area, A_NORMAL);
		}
		else if(maxX == lengths[k]) {
			printf_xy( (int) (cos(pAngles[i]) * pLengths[j] * ratio) + 5,60,"%c",pName[k]);
			printf_xy( (int) 5, 92, "%c", pName[i]);
			printf_xy( (int) (maxX * ratio) + 5, 92, "%c", pName[j]);
			FillTriangle ((int) (cos(pAngles[i]) * pLengths[j] * ratio) + 5, 68,
										5, 92,
										(int) (maxX * ratio) + 5, 92, &area, A_NORMAL);
		}
		else {
			printf_xy( (int) (maxX * ratio) + 5,60,"%c",pName[k]);
			printf_xy( (int) 5, 92, "%c", pName[i]);
			printf_xy( (int) (pLengths[k] * ratio) + 5, 92, "%c", pName[j]);
			FillTriangle ( (int) (maxX * ratio) + 5, 68,
										5, 92,
										(int) (pLengths[k] * ratio) + 5, 92, &area, A_NORMAL);
		}
	}
	else {
		ratio = 150 / (maxX - minX);
		if(minX < 0) {
			printf_xy( 5, 84 - (int) (maxY * ratio),"%c",pName[k]);
			printf_xy( (int) (-minX * ratio), 92, "%c", pName[j]);
			printf_xy( (int) ((-minX + pLengths[k]) * ratio ), 92, "%c", pName[i]);
			FillTriangle (5, 92 - (int) (maxY * ratio),
										(int) (-minX * ratio) + 5, 92,
										(int) ((-minX + pLengths[k]) * ratio ) + 5, 92, &area, A_NORMAL);
		}
		else if(maxX == lengths[k]) {
			printf_xy( (int) (cos(pAngles[i]) * pLengths[j] * ratio), 84 - (int) (maxY * ratio),"%c",pName[k]);
			printf_xy( (int) 5, 92, "%c", pName[i]);
			printf_xy( (int) (maxX * ratio), 92, "%c", pName[j]);
			FillTriangle ((int) (cos(pAngles[i]) * pLengths[j] * ratio) + 5, 92 - (int) (maxY * ratio),
										5, 92,
										(int) (maxX * ratio) + 5, 92, &area, A_NORMAL);
		}
		else {
			printf_xy( (int) (maxX * ratio),84 - (int) (maxY * ratio),"%c",pName[k]);
			printf_xy( (int) 5, 92, "%c", pName[i]);
			printf_xy( (int) (pLengths[k] * ratio), 92, "%c", pName[j]);
			FillTriangle ( (int) (maxX * ratio) + 5, 92 - (int) (maxY * ratio),
										5, 92,
										(int) (pLengths[k] * ratio) + 5, 92, &area, A_NORMAL);
		}
	}
}

void setup_program(void) {
	// Setup the queue
	kbq = kbd_queue();

	// Setup state machine
	state = 0;
	prep_state = -1;
	
	// Set font size
	FontSetSys (F_6x8);
	
	// Blank the screen
	ClrScr();
	
	menu_handle = MenuNew (MF_TOOLBOX, 160, 18);
	MenuAddText (menu_handle, -1, "Quit", 1, DMF_TOP);
	MenuAddText (menu_handle, -1, "Restart", 2, DMF_TOP);
	MenuAddText (menu_handle, -1, "Update Pieces", 3, DMF_TOP);
	exec_handle = MenuBegin (NULL, 0, 0, 0x20, menu_handle);
	
	// Take this snapshot so we don't need to redraw this constantly
	LCD_save (&UI);
}

